/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.creator;

import java.io.File;
import org.apache.log4j.Logger;

import com.sos.JSHelper.Basics.JSToolBox;
import com.sos.jobnet.classes.JobNetConstants;
import com.sos.jobnet.classes.JobNetException;
import com.sos.jobnet.creator.jobs.JobNetCreatorOptions;
import com.sos.jobnet.db.JobNetEdgesDBItem;
import com.sos.jobnet.db.JobNetEdgesDBLayer;
import com.sos.jobnet.db.JobNetNodeDBItem;
import com.sos.jobnet.db.JobNetNodeDBLayer;
import com.sos.localization.Messages;

/**
 * \file JobNetCreator.java
 * \brief creating a jobnet 
 *  
 * \class JobNetCreator
 * \brief creating a jobnet 
 * 
 * \details
 * This class is for creating a jobnet. A jobnet contains of a list of nodes connected by edges. Each jobnet has a special node called
 * bootstrap node (bootstrap order). The bootstrap node marks the starting point of the jobnet.
 * 
 * By calling the run method the complete jobnet (a collection of JobScheduler orders) will read from the filesystem and stored in the
 * database in the tables SCHEDULER_JOB_NET_NODES and SCHEDULER_JOB_NET_EDGES.
 * \code
  \endcode
 *
 * \author Stefan Schaedlich
 * \version 1.0 - 05.03.2012 16:24:03
 * <div class="sos_branding">
 *   <p> 2010 SOS GmbH - Berlin (<a style='color:silver' href='http://www.sos-berlin.com'>http://www.sos-berlin.com</a>)</p>
 * </div>
 */
public class JobNetCreator implements Runnable {
	
	@SuppressWarnings("unused")
	private static final String conSVNVersion = "$Id: JobNetCreator.java 19153 2013-02-19 13:38:55Z ss $";
	
	private final static Logger logger = Logger.getLogger(JobNetCreator.class);

	private static final Messages msg = new JSToolBox(JobNetConstants.strBundleBaseName).getMessageObject();
	private static final String	JOBNETC_I_0001 = "JOBNETC_I_0001";		// create job net for order %1$s (jobchain %2$s).
	private static final String JOBNETC_I_0002 = "JOBNETC_I_0002"; 		// %1$s nodes readed from file system.
	private static final String JOBNETC_I_0003 = "JOBNETC_I_0003";		// jobnet successfully written in database.
	private static final String JOBNETC_I_0005 = "JOBNETC_I_0005";		// the texture of the jobnet will not be checked.
	private static final String JOBNETC_I_0006 = "JOBNETC_I_0006";		// checking the texture of the jobnet.

//	private static final String JOBNETC_E_0001 = "JOBNETC_E_0001"; 		// %1$s is successor of %2$s but %3$s is not predecessor of %4$s
//	private final static String JOBNETC_E_0002 = "JOBNETC_E_0002"; 		// %1$s is predecessor of %2$s but %3$s is not successor of %4$s.

	private final static String JOBNETC_E_0014 = "JOBNETC_E_0014"; 		// error creating jobnet
	
	private final JobNetCreatorOptions options;
	private final ObjectModelConnector connection;
	
	private final String orderId;
	private final String jobChain;
	private final String clusterId;
	private final int maxNodes;
	private final boolean checkJobNetTexture;
	private final boolean createGraph;
	private final JobNetToGraphviz graph;
	private JobNetBootstrapOrder bootstrapOrder;
	
	public JobNetCreator(JobNetCreatorOptions options) {
		this.options = options;
		this.orderId = options.OrderId.Value();
		this.jobChain = options.JobChainName.Value();
		this.clusterId = options.SCHEDULER_ID.Value();
		this.checkJobNetTexture = options.CheckJobNetTexture.value();
		this.maxNodes = Integer.parseInt(options.getMaxJobNetNodes().Value());
		this.createGraph = options.CreateGraph.value();
		this.graph = new JobNetToGraphviz(options);
		
		connection = ObjectModelConnector.getInstance(options.SCHEDULER_HOST, options.SCHEDULER_TCP_PORT, options.SCHEDULER_CONFIGURATION_DIRECTORY);
	}

	@Override
	public void run() {
		
		try {
			logger.info(msg.getMsg(JOBNETC_I_0001,orderId,jobChain));
			bootstrapOrder = new JobNetBootstrapOrder(connection,connection.getRootDirectory(),clusterId,orderId,jobChain,maxNodes);
			bootstrapOrder.createJobNet();
			
			if (checkJobNetTexture) {
				logger.info(msg.getMsg(JOBNETC_I_0006));
				bootstrapOrder.validateJobNet();
			} else {
				logger.info(msg.getMsg(JOBNETC_I_0005));
			}
			
			logger.info(msg.getMsg(JOBNETC_I_0002,getBootstrapOrder().getJobNet().size()));
			writeToDB();
			
			if (createGraph) {
				if (getBootstrapOrder().getJobNet().size() == 1) {
					logger.warn("The jobnet contains only one node therefore no graph can be generated.");
				} else {
					logger.info("graph will be created");
					graph.createGraph(bootstrapOrder);
				}
			}
			
			logger.info(msg.getMsg(JOBNETC_I_0003));

		} catch (Exception e) {
			String msgText = msg.getMsg(JOBNETC_E_0014);
			logger.error(msgText);
			throw new JobNetException(msgText,e);
		}
		
	}
	
	private void writeToDB() {

		File configFile = new File(options.hibernate_connection_config_file.Value());
		JobNetNodeDBLayer nodeDbLayer = new JobNetNodeDBLayer(configFile,clusterId);
		JobNetEdgesDBLayer edgeDbLayer = new JobNetEdgesDBLayer(configFile);
		try {
			nodeDbLayer.beginTransaction();
			edgeDbLayer.beginTransaction();		// is neccessary for writing the edges
	
			for(JobNetOrder o : bootstrapOrder.getJobNet().values()) {
				JobNetOrderDBAdapter adapter = new JobNetOrderDBAdapter(nodeDbLayer, edgeDbLayer, o);
				if (!adapter.itemExists()) {
					JobNetNodeDBItem nodeItem = adapter.createJobNetNode();
					nodeDbLayer.save(nodeItem);
					o.setNodeDBItem(nodeItem);
				} else {
					o.setNodeDBItem(adapter.getItem());
				}
			}
	
			for(JobNetOrder o : bootstrapOrder.getJobNet().values()) {
				JobNetOrderDBAdapter adapter = new JobNetOrderDBAdapter(nodeDbLayer, edgeDbLayer, o);
				adapter.deleteAllEdges();
			}
			
			JobNetEdgesDBItem edgeItem;
			for(JobNetOrder parent : bootstrapOrder.getJobNet().values()) {
				JobNetOrderDBAdapter parentAdapter = new JobNetOrderDBAdapter(nodeDbLayer, edgeDbLayer, parent);
				if (parentAdapter.itemExists()) {
					for(JobNetOrder child : parent.getSuccessors()) {
						JobNetOrderDBAdapter childAdapter = new JobNetOrderDBAdapter(nodeDbLayer, edgeDbLayer, child);
						if (childAdapter.itemExists()) {
							edgeItem = parentAdapter.createSuccessor(childAdapter.getItem());
							edgeDbLayer.save(edgeItem);
							edgeItem = childAdapter.createPredecessor(parentAdapter.getItem());
							edgeDbLayer.save(edgeItem);
						}
					}
				}
			}
			nodeDbLayer.commit();		// commits implicitly the edges table
		} finally {
			nodeDbLayer.closeSession();
			edgeDbLayer.closeSession();
		}
	}
	
	public JobNetCreatorOptions getOptions() {
		return options;
	}

	public JobNetBootstrapOrder getBootstrapOrder() {
		return bootstrapOrder;
	}
	
}
