/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.creator;

import java.io.File;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.joda.time.Interval;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import com.sos.JSHelper.Basics.JSToolBox;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.jobnet.classes.JobNetConstants;
import com.sos.jobnet.classes.JobNetException;
import com.sos.jobnet.classes.Status.enuStatus;
import com.sos.jobnet.creator.jobs.JobNetCreatorOptions;
import com.sos.jobnet.db.JobNetPlanDBItem;
import com.sos.jobnet.db.JobNetPlanDBLayer;
import com.sos.jobnet.db.JobNetPlanFilter;
import com.sos.localization.Messages;
import com.sos.scheduler.model.objects.JSObjRunTime;

/**
 * \file JobNetPlanCreator.java
 * \brief creating a jobnet plan 
 *  
 * \class JobNetPlanCreator
 * \brief creating a jobnet plan
 * 
 * \details
 * This class assumes a complete jobnet stored in the database for the bootstrap order which the class is 
 * called for (see com.sos.jobnet.creator.JobNetCreator). Therefore the first thing calling the run method
 * is to execute the creation of the jobnet with the run method of com.sos.jobnet.creator.JobNetCreator.
 *
 * Based on this jobnet and the order definitions in the JobScheduler configuration a jobnet plan is created and
 * stored in the database in table SCHEDULER_JOB_NET_PLAN. Among other things the jobnet plan contains the 
 * complete configuration of the order. This is the base to execute the jobnet.
 * \code
  \endcode
 *
 * \author Stefan Schdlich
 * \version 1.0 - 03.04.2012 10:12:21
 * <div class="sos_branding">
 *   <p> 2010 SOS GmbH - Berlin (<a style='color:silver' href='http://www.sos-berlin.com'>http://www.sos-berlin.com</a>)</p>
 * </div>
 */
public class JobNetPlanCreator extends JobNetCreator implements Runnable {
	
	@SuppressWarnings("unused")
	private static final String conSVNVersion = "$Id: JobNetPlanCreator.java 19342 2013-03-05 09:39:48Z ss $";

	private final static Logger logger = Logger.getLogger(JobNetPlanCreator.class);

	private final static DateTimeFormatter fmtDateTime = DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss");
//	private final static DateTime now = new DateTime();
//	private final static DateTime onDemand = now.minusMillis(now.getMillisOfDay());

	private static final Messages msg = new JSToolBox(JobNetConstants.strBundleBaseName).getMessageObject();
	private final static String JOBNETC_I_0010 = "JOBNETC_I_0010"; // job net plan created for order %1$s.
	private final static String JOBNETC_I_0012 = "JOBNETC_I_0012"; // creating jobnet with uuid %1$s.
//	private final static String JOBNETC_W_0001 = "JOBNETC_W_0001"; // the period %1$s does not represent a single start.
//	private final static String JOBNETC_E_0006 = "JOBNETC_E_0006"; // the bootstrap order %1$s contains no single_start.
	private final static String JOBNETC_E_0007 = "JOBNETC_E_0007"; // starttime %1$s is not parsable.
	private final static String JOBNETC_E_0008 = "JOBNETC_E_0008"; // the external uuid already exists.
	private final static String JOBNETC_E_0009 = "JOBNETC_E_0009"; // more than one starttime for an external uuid.
	private final static String JOBNETC_E_0015 = "JOBNETC_E_0015"; // error creating the jobnet plan

	private final static int NOT_PROCESSED = enuStatus.NOT_PROCESSED.ordinal();
	
	private final Interval interval4StartTimes;
	private final boolean startOrder;
	private final boolean checkFrequency;
	private final String externalUUID;
	private final JobNetPlanExecutor runner;
	private final FrequencyChecker checker;

	private List<JobNetPlanDBItem> plannedOrders = new ArrayList<JobNetPlanDBItem>();

	public JobNetPlanCreator(JobNetCreatorOptions options) {
		super(options);
		DateTime from = new DateTime();
		DateTime to = new DateTime(options.gettime_horizon().getEndFromNow());
		this.interval4StartTimes = new Interval(from,to);
		this.startOrder = options.getStartOrder().value();
		this.checkFrequency = options.getCheckFrequency().value();
		this.runner =  new JobNetPlanExecutor(options);
		this.checker = new FrequencyChecker(options);
		this.externalUUID = options.getJobNetIdPrefix().Value() + options.getJobNetId().Value();
		logger.info("Timeinterval to search for start times: " + fmtDateTime.print(from) + " to " + fmtDateTime.print(to));
	}

	@Override
	public void run() {
		
		try {
			super.run(); // ensures the representation of the jobNet in the database
	
			/*
			 * In case of an external given uuid it has to be unique.
			 */
			File configFile = new File(getOptions().hibernate_connection_config_file.Value());
			JobNetPlanDBLayer planDbLayer = new JobNetPlanDBLayer(configFile, getOptions().SCHEDULER_ID.Value());
			if (hasExternalUUID()) {
				logger.info("Creating jobnnet instance with external id " + externalUUID);
				if (isJobNetInstance(planDbLayer, externalUUID)) {
					String msgText = msg.getMsg(JOBNETC_E_0008, externalUUID);
					logger.error(msgText);
					throw new JobSchedulerException(msgText);
				}
			}
			
			JobNetBootstrapOrder bootstrapOrder = getBootstrapOrder();
			
			JSObjRunTime runTime = bootstrapOrder.getJSObjRunTime();
			planDbLayer.beginTransaction();
			
			JobNetPlanDBItem bootDBItem = null;
			if (JobNetOrder.isOnDemandOrder(bootstrapOrder)) {
				bootDBItem = createJobNetPlan4DemandOrderInDB(planDbLayer, createUUID());
				logger.info("Order " + bootstrapOrder.getJobNetNodeName() + " will be start on demand");
				plannedOrders.add(bootDBItem);
			} else {
				if (runTime.hasSubsequentRunTimes()) {
					
					/*
					 * If the bootstrap order contains more than one start time, it is not possible to set the uuid external.
					 */
					List<DateTime> startTimes = runTime.getDtSingleStarts(interval4StartTimes);
					logger.info(startTimes.size() + " runtimes calculated for " + bootstrapOrder.getJobNetNodeName());
					if (hasExternalUUID() && startTimes.size() > 1) {
						String msgText = msg.getMsg(JOBNETC_E_0009);
						logger.error(msgText);
						throw new JobSchedulerException(msgText);
					}
					
					for (DateTime startTime : startTimes) {
						String startString = fmtDateTime.print(startTime);
						bootDBItem = createJobNetPlan4OrderInDB(planDbLayer, createUUID(), startString);
						logger.info("Order " + bootstrapOrder.getJobNetNodeName() + " will be start at " + startString);
						plannedOrders.add(bootDBItem);
					}
					
				} else {
					bootDBItem = createJobNetPlan4OrderInDB(planDbLayer, createUUID(), "now");
					logger.info("Order " + bootstrapOrder.getJobNetNodeName() + " will be start immediately");
					plannedOrders.add(bootDBItem);
				}
			}

			planDbLayer.commit();
			planDbLayer.closeSession();
		} catch (Exception e) {
			String msgText = msg.getMsg(JOBNETC_E_0015);
			logger.error(msgText);
			throw new JobSchedulerException(msgText,e);
		}
		
		if (checkFrequency)
			for (JobNetPlanDBItem planItem : getPlannedOrders())
				checker.processJobNet(planItem);
		
		if (startOrder) {
			for (JobNetPlanDBItem planItem : getPlannedOrders())
				runner.startOrder(planItem, getBootstrapOrder());
		}

	}
	
	private String createUUID() {
		return (hasExternalUUID()) ? externalUUID : UUID.randomUUID().toString();
	}
	
	private JobNetPlanDBItem createJobNetPlan4DemandOrderInDB(JobNetPlanDBLayer planDbLayer, String uuid) {
		return createJobNetPlanAndWrite2Db(planDbLayer, uuid, "now", true);
	}
	
	private JobNetPlanDBItem createJobNetPlan4OrderInDB(JobNetPlanDBLayer planDbLayer, String uuid, String startTime) {
		return createJobNetPlanAndWrite2Db(planDbLayer, uuid, startTime, false);
	}
	
	private JobNetPlanDBItem createJobNetPlanAndWrite2Db(JobNetPlanDBLayer planDbLayer, String uuid, String startTime, boolean isOnDemand) {
		JobNetPlanDBItem bootStrap = null;
		logger.info(msg.getMsg(JOBNETC_I_0012,uuid));
		for (JobNetOrder order : getBootstrapOrder().getJobNet().values()) {
			JobNetPlanDBItem item = new JobNetPlanDBItem();
			item.setUuid(uuid);
			item.setStatusNode(NOT_PROCESSED);
			item.setStatusWaiter(NOT_PROCESSED);
			item.setStatusRunner(NOT_PROCESSED);
			item.setStatusDispatcher(NOT_PROCESSED);
			item.setOrderXml(order.toXMLString());
			item.setJobnetNodeDBItem(order.getNodeDBItem());
			item.setPlanId(order.getNodeDBItem().getNodeId());
			item.setCreated(new Date());
			item.setCreatedBy(JobNetPlanCreator.class.getName());
			item.setModified(new Date());
			item.setModifiedBy(JobNetPlanCreator.class.getName());

			item.setExitCode(0L);
			item.setFrequencyOverwrite(false);
			item.setIsWaiterSkipped(false);
			item.setIsRunnerSkipped(false);
			item.setIsDispatcherSkipped(false);
			item.setIsRunnerOnDemand(false);		// is set by FrequenyChecker (except for bootstarp order)
			item.setBootstrap(order.isBootstrap());
		
			if (order.isBootstrap()) {
				item.setIsRunnerOnDemand(isOnDemand);
				try {
					item.setPlannedStartime(startTime);
				} catch (ParseException e) {
					String msgText = msg.getMsg(JOBNETC_E_0007,order.getAt());
					logger.error(msgText,e);
					throw new JobNetException(msgText);
				}
				bootStrap = item;
			}
			planDbLayer.save(item);
			if (order.isBootstrap()) logger.info(msg.getMsg(JOBNETC_I_0010,order.getJobNetNodeName()));
		}
		return bootStrap;
	}

	public List<JobNetPlanDBItem> getPlannedOrders() {
		return plannedOrders;
	}
	
	private boolean isJobNetInstance(JobNetPlanDBLayer planDbLayer, String uuid) {
		JobNetPlanFilter filter = new JobNetPlanFilter();
		filter.setUuid(uuid);
		planDbLayer.setFilter(filter);
		int result = planDbLayer.getJobnetPlanList(0).size();
		logger.info("The jobnet with id " + uuid + " contains " + result + " nodes.");
		return (result != 0);
	}
	
	private boolean hasExternalUUID() {
		return (!externalUUID.isEmpty());
	}
	
}
